# btsgtk/batch.py
#  Copyright (C) 2008 Luca Bruno <lethalman88@gmail.com>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; see the file COPYING.  If not, write to
#  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
#  Boston, MA 02111-1307, USA.

import gtk

from bug import Bug
import btsop

class ControllerPlate (gtk.HBox):
    def __init__ (self, application, controller):
        gtk.HBox.__init__ (self, spacing=6)
        self.application = application
        self.controller = controller

        self.create_label ()
        self.create_choice ()
        self.create_addremove ()
        self.show ()

    def create_label (self):
        label = gtk.Label ("Operation:")
        label.show ()
        self.pack_start (label, expand=False)

    def create_choice (self):
        self.model = gtk.ListStore (str, str, object)
        self.combo = gtk.ComboBox (self.model)

        cell = gtk.CellRendererText ()
        self.combo.pack_start (cell)
        self.combo.add_attribute (cell, 'text', 0)

        cell = gtk.CellRendererText ()
        self.combo.pack_start (cell)
        self.combo.add_attribute (cell, 'text', 1)

        for btsop_class in btsop.operations:           
            self.model.append ([btsop_class.name, btsop_class.description, btsop_class])

        self.combo.connect ('changed', self.on_changed)
        self.combo.show ()
        self.pack_start (self.combo)

    def create_addremove (self):
        self.button = gtk.Button ("")
        self.button.connect ('clicked', self.on_delete)
        self.button.hide ()
        self.pack_start (self.button)

    def setup (self, btsop):
        if not btsop:
            self.controller.cleared ()
        else:
            for row in self.model:
                if row[2] == btsop.__class__:
                    self.combo.set_active_iter (row.iter)
            self.controller.filled ()

    def on_changed (self, combo):
        iter = self.combo.get_active_iter ()
        if not iter:
            return
        btsop_class = self.model.get_value (iter, 2)
        btsop = btsop_class (self.application)
        self.button.show ()
        image = gtk.image_new_from_stock (gtk.STOCK_DELETE, gtk.ICON_SIZE_BUTTON)
        self.button.set_image (image)
        self.controller.setup (btsop)

    def on_delete (self, button):
        self.controller.cleared ()

class BatchController (gtk.VBox):
    def __init__ (self, application, batch, bug):
        gtk.VBox.__init__ (self, spacing=6)
        self.application = application
        self.batch = batch
        self.bug = bug
        self.btsop = None

        self.create_plate ()
        self.create_placeholder ()
        self.show ()

    def create_plate (self):
        self.plate = ControllerPlate (self.application, self)
        self.pack_start (self.plate, expand=False)
        
    def create_placeholder (self):
        self.placeholder = gtk.EventBox ()
        self.placeholder.show ()
        self.pack_start (self.placeholder)

    def setup (self, btsop):
        self.btsop = btsop
        self.plate.setup (btsop)

    def drop_child (self):
        child = self.placeholder.get_child ()
        if child:
            self.placeholder.remove (child)
            child.destroy ()

    def cleared (self):
        self.is_setup = False
        self.drop_child ()
        self.batch.remove_controller (self)
        
    def filled (self):
        self.is_setup = True
        self.drop_child ()
        self.btsop.setup (self.bug)
        self.placeholder.add (self.btsop)
        # Add a new empty controller
        self.batch.add_operation (None)

    def execute (self):
        if not self.is_setup:
            return
        self.btsop.execute ()

class Batch (gtk.VBox):
    def __init__ (self, application, number):
        gtk.VBox.__init__ (self, spacing=12)
        self.application = application
        self.bug = Bug (application, number)
        self.controllers = []

        self.create_label ()
        self.create_list ()
        self.add_operation (None)

        self.show ()

    def get_bug_number (self):
        return self.bug.get_number ()

    def create_label (self):
        label = gtk.Label (self.bug)
        label.show ()
        self.pack_start (label, expand=False)

    def create_list (self):
        scrolled = gtk.ScrolledWindow ()
        scrolled.set_policy (gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        scrolled.set_shadow_type (gtk.SHADOW_ETCHED_IN)
        viewport = gtk.Viewport ()
        viewport.show ()
        self.listbox = gtk.VBox (spacing=24)
        self.listbox.show ()
        viewport.add (self.listbox)
        scrolled.add (viewport)
        scrolled.show ()
        self.pack_start (scrolled)

    def get_last_controller (self):
        if self.controllers:
            return self.controllers[-1]

    def add_operation (self, btsop):
        "A None btsop means an empty controller waiting for user input"

        # Do not create a new controller if the last one is already empty
        last = self.get_last_controller ()
        if last and not last.is_setup:
            last.setup (btsop)
            return last

        controller = BatchController (self.application, self, self.bug)
        self.controllers.append (controller)
        self.listbox.pack_start (controller, expand=False)
        controller.setup (btsop)

    def remove_controller (self, controller):
        # Do not remove the last controller, it can be setup twice
        last = self.get_last_controller ()
        if last == controller:
            return

        self.controllers.remove (controller)
        self.listbox.remove (controller)
        controller.unparent ()
        controller.destroy ()

    def execute (self):
        for controller in self.controllers:
            controller.execute ()
